searchData={"items":[{"type":"module","title":"Gnat","doc":"The primary interface for interacting with NATS","ref":"Gnat.html"},{"type":"function","title":"Gnat.active_subscriptions/1","doc":"Get the number of active subscriptions","ref":"Gnat.html#active_subscriptions/1"},{"type":"function","title":"Gnat.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Gnat.html#child_spec/1"},{"type":"type","title":"Gnat.connection_settings/0","doc":"* `connection_timeout` - limits how long it can take to establish a connection to a server\n* `host` - The location of the NATS server\n* `ping_interval` - The number of milliseconds between sending PING messages to the server to check the health of our connection\n* `port` - The port the NATS server is listening on\n* `ssl_opts` - Options for connecting over SSL\n* `tcp_opts` - Options for connecting over TCP\n* `tls` - If the server should use a TLS connection\n* `inbox_prefix` - Prefix to use for the message inbox of this connection\n* `no_responders` - Enable the no responders behavior (see `Gnat.request/4`)","ref":"Gnat.html#t:connection_settings/0"},{"type":"type","title":"Gnat.headers/0","doc":"","ref":"Gnat.html#t:headers/0"},{"type":"type","title":"Gnat.message/0","doc":"A message received from NATS will be delivered to your process in this form.\nPlease note that the `:reply_to` and `:headers` keys are optional.\nThey will only be present if the message was received from the NATS server with\nheaders or a `reply_to` topic\n\n* `gnat` - The Gnat connection\n* `topic` - The topic on which the message arrived\n* `body` - The raw payload of the message\n* `sid` - The subscription ID corresponding to this message. You generally won't need to use this value directly.\n* `reply_to` - A topic supplied for expected replies\n* `headers` - A set of NATS message headers on the message\n* `status` - Similar to an HTTP status, this is present for messages with headers and can indicate the specific purpose of a message. Example `status: \"408\"`\n* `description` - A string description of the `status`","ref":"Gnat.html#t:message/0"},{"type":"function","title":"Gnat.ping/1","doc":"Kept just for backward compatibility for now","ref":"Gnat.html#ping/1"},{"type":"function","title":"Gnat.pub/4","doc":"Publish a message\n\n```\n{:ok, gnat} = Gnat.start_link()\n:ok = Gnat.pub(gnat, \"characters\", \"Ron Swanson\")\n```\n\nIf you want to provide a reply address to receive a response you can pass it as an option.\n[See request-reply pattern](https://docs.nats.io/nats-concepts/core-nats/reqreply).\n\n```\n{:ok, gnat} = Gnat.start_link()\n:ok = Gnat.pub(gnat, \"characters\", \"Star Lord\", reply_to: \"me\")\n```\n\nIf you want to publish a message with headers you can pass the `:headers` key in the `opts` like this.\n\n```\n{:ok, gnat} = Gnat.start_link()\n:ok = Gnat.pub(gnat, \"listen\", \"Yo\", headers: [{\"foo\", \"bar\"}])\n```\n\nHeaders must be passed as a `t:headers()` value (a list of tuples).\nSending and parsing headers has more overhead than typical nats messages\n(see [the Nats 2.2 release notes for details](https://docs.nats.io/whats_new_22#message-headers)),\nso only use them when they are really valuable.","ref":"Gnat.html#pub/4"},{"type":"function","title":"Gnat.request/4","doc":"Send a request and listen for a response synchronously\n\nFollowing the nats [request-reply pattern](https://docs.nats.io/nats-concepts/core-nats/reqreply) this\nfunction generates a one-time topic to receive replies and then sends a message to the provided topic.\n\nSupported options:\n  * `receive_timeout` - An integer number of milliseconds to wait for a response. Defaults to 60_000\n  * `headers` - A set of headers you want to send with the request (see `Gnat.pub/4`)\n\n```\n{:ok, gnat} = Gnat.start_link()\ncase Gnat.request(gnat, \"i_can_haz_cheezburger\", \"plZZZZ?!?!?\") do\n  {:ok, %{body: delicious_cheezburger}} -> :yum\n  {:error, :timeout} -> :sad_cat\nend\n```","ref":"Gnat.html#request/4"},{"type":"function","title":"No Responders - Gnat.request/4","doc":"If you send a request to a topic that has no registered listeners, it is sometimes convenient to find out\nright away, rather than waiting for a timeout to occur. In order to support this use-case, you can start\nyour Gnat connection with the `no_responders: true` option and this function will return very quickly with\nan `{:error, :no_responders}` value. This behavior also works with `request_multi/4`","ref":"Gnat.html#request/4-no-responders"},{"type":"function","title":"Gnat.request_multi/4","doc":"Send a request and listen for multiple responses synchronously\n\nThis function makes it easy to do a scatter-gather operation where you wait for a limited time\nand optionally a maximum number of replies.\n\nSupported options:\n  * `receive_timeout` - An integer number of milliseconds to wait for responses. Defaults to 60_000\n  * `max_messages` - An integer number of messages to listen for. Defaults to -1 meaning unlimited\n  * `headers` - A set of headers you want to send with the request (see `Gnat.pub/4`)\n\n```\n{:ok, gnat} = Gnat.start_link()\n{:ok, responses} = Gnat.request_multi(gnat, \"i_can_haz_fries\", \"plZZZZZ!?!?\", max_messages: 5)\nEnum.count(responses) #=> 5\n```","ref":"Gnat.html#request_multi/4"},{"type":"type","title":"Gnat.sent_message/0","doc":"","ref":"Gnat.html#t:sent_message/0"},{"type":"type","title":"Gnat.server_info/0","doc":"[Info Protocol](https://docs.nats.io/reference/reference-protocols/nats-protocol#info)\n\n* `client_id` - An optional unsigned integer (64 bits) representing the internal client identifier in the server. This can be used to filter client connections in monitoring, correlate with error logs, etc...\n* `client_ip` - The IP address the client is connecting from\n* `cluster` - The name of the cluster if any\n* `cluster_dynamic` - If the cluster is dynamic\n* `connect_urls` - An optional list of server urls that a client can connect to.\n* `ws_connect_urls` - An optional list of server urls that a websocket client can connect to.\n* `git_commit` - The git commit associated with this NATS version\n* `go` - The version of golang the NATS server was built with\n* `headers` - If messages can have headers in them\n* `host` - The IP address used to start the NATS server, by default this will be 0.0.0.0 and can be configured with -client_advertise host:port\n* `jetstream` - If the server is using JetStream features\n* `max_payload` - Maximum payload size, in bytes, that the server will accept from the client\n* `port` - The port number the NATS server is configured to listen on\n* `proto` - An integer indicating the protocol version of the server. The server version 1.2.0 sets this to 1 to indicate that it supports the \"Echo\" feature.\n* `server_id` - The unique identifier of the NATS server\n* `server_name` - A name for the server\n* `version` - The version of the NATS server\n* `ldm` - If the server supports Lame Duck Mode notifications, and the current server has transitioned to lame duck, ldm will be set to true.\n* `auth_required` - If this is set, then the client should try to authenticate upon connect.\n* `tls_required` - If this is set, then the client must perform the TLS/1.2 handshake. Note, this used to be ssl_required and has been updated along with the protocol from SSL to TLS.\n* `tls_verify` - If this is set, the client must provide a valid certificate during the TLS handshake.\n* `tls_available` - If the server can use TLS","ref":"Gnat.html#t:server_info/0"},{"type":"function","title":"Gnat.server_info/1","doc":"Get information about the NATS server the connection is for","ref":"Gnat.html#server_info/1"},{"type":"function","title":"Gnat.start_link/2","doc":"Starts a connection to a nats broker\n\n```\n{:ok, gnat} = Gnat.start_link(%{host: \"127.0.0.1\", port: 4222})\n# if the server requires TLS you can start a connection with:\n{:ok, gnat} = Gnat.start_link(%{host: \"127.0.0.1\", port: 4222, tls: true})\n# if the server requires TLS and a client certificate you can start a connection with:\n{:ok, gnat} = Gnat.start_link(%{tls: true, ssl_opts: [certfile: \"client-cert.pem\", keyfile: \"client-key.pem\"]})\n# you can customize default \"_INBOX.\" inbox prefix with:\n{:ok, gnat} = Gnat.start_link(%{host: \"127.0.0.1\", port: 4222, inbox_prefix: \"my_prefix._INBOX.\"})\n# you can use IPv6 addresses too\n{:ok, gnat} = Gnat.start_link(%{host: \"::1\", port: 4222, tcp_opts: [:inet6, :binary]})\n```\n\nYou can also pass arbitrary SSL or TCP options in the `tcp_opts` and `ssl_opts` keys.\nIf you pass custom TCP options please include `:binary`. Gnat uses binary matching to parse messages.\n\nThe final `opts` argument will be passed to the `GenServer.start_link` call so you can pass things like `[name: :gnat_connection]`.","ref":"Gnat.html#start_link/2"},{"type":"function","title":"Gnat.stop/1","doc":"Gracefully shuts down a connection\n\n```\n{:ok, gnat} = Gnat.start_link()\n:ok = Gnat.stop(gnat)\n```","ref":"Gnat.html#stop/1"},{"type":"function","title":"Gnat.sub/4","doc":"Subscribe to a topic\n\nSupported options:\n  * queue_group: a string that identifies which queue group you want to join\n\nBy default each subscriber will receive a copy of every message on the topic.\nWhen a queue_group is supplied messages will be spread among the subscribers\nin the same group. (see [nats queueing](https://nats.io/documentation/concepts/nats-queueing/))\n\nThe subscribed process will begin receiving messages with a structure of `t:sent_message/0`\n\n```\n{:ok, gnat} = Gnat.start_link()\n{:ok, subscription} = Gnat.sub(gnat, self(), \"topic\")\nreceive do\n  {:msg, %{topic: \"topic\", body: body}} ->\n    IO.puts \"Received: #{body}\"\nend\n```","ref":"Gnat.html#sub/4"},{"type":"type","title":"Gnat.t/0","doc":"","ref":"Gnat.html#t:t/0"},{"type":"function","title":"Gnat.unsub/3","doc":"Unsubscribe from a topic\n\nSupported options:\n  * `max_messages` - Number of messages to be received before automatically unsubscribed\n\nThis correlates to the [UNSUB](https://docs.nats.io/reference/reference-protocols/nats-protocol#unsub) command in the nats protocol.\nBy default the unsubscribe is affected immediately, but an optional `max_messages` value can be provided which will allow\n`max_messages` to be received before affecting the unsubscribe.\nThis is especially useful for request/reply patterns.\n\n```\n{:ok, gnat} = Gnat.start_link()\n{:ok, subscription} = Gnat.sub(gnat, self(), \"my_inbox\")\n:ok = Gnat.unsub(gnat, subscription)\n# OR\n:ok = Gnat.unsub(gnat, subscription, max_messages: 2)\n```","ref":"Gnat.html#unsub/3"},{"type":"module","title":"Gnat.ConnectionSupervisor","doc":"A process that can supervise a named connection for you\n\nIf you would like to supervise a Gnat connection and have it automatically re-connect in case of failure you can use this module in your supervision tree.\nIt takes a map with the following data:\n\n```\ngnat_supervisor_settings = %{\n  name: :gnat, # (required) the registered named you want to give the Gnat connection\n  backoff_period: 4_000, # number of milliseconds to wait between consecutive reconnect attempts (default: 2_000)\n  connection_settings: [\n    %{host: '10.0.0.100', port: 4222},\n    %{host: '10.0.0.101', port: 4222},\n  ]\n}\n```\n\nThe connection settings can specify all of the same values that you pass to `Gnat.start_link/1`. Each time a connection is attempted we will use one of the provided connection settings to open the connection. This is a simplistic way of load balancing your connections across a cluster of nats nodes and allowing failover to other nodes in the cluster if one goes down.\n\nTo use this in your supervision tree add an entry like this:\n\n```\nimport Supervisor.Spec\nworker(Gnat.ConnectionSupervisor, [gnat_supervisor_settings, [name: :my_connection_supervisor]])\n```\n\nThe second argument is used as GenServer options so you can give the supervisor a registered name as well if you like. Now in the rest of your code you can call things like:\n\n```\n:ok = Gnat.pub(:gnat, \"subject\", \"message\")\n```\n\nAnd it will use your supervised connection. If the connection is down when you call that function (or dies during that function) it will raise an error.","ref":"Gnat.ConnectionSupervisor.html"},{"type":"function","title":"Gnat.ConnectionSupervisor.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Gnat.ConnectionSupervisor.html#child_spec/1"},{"type":"function","title":"Gnat.ConnectionSupervisor.start_link/2","doc":"","ref":"Gnat.ConnectionSupervisor.html#start_link/2"},{"type":"module","title":"Gnat.ConsumerSupervisor","doc":"A process that can supervise consumers for you\n\nIf you want to subscribe to a few topics and have that subscription last across restarts for you, then this worker can be of help.\nIt also spawns a supervised `Task` for each message it receives.\nThis way errors in message processing don't crash the consumers, but you will still get SASL reports that you can send to services like honeybadger.\n\nTo use this just add an entry to your supervision tree like this:\n\n```\nconsumer_supervisor_settings = %{\n  connection_name: :name_of_supervised_connection,\n  module: MyApp.Server, # a module that implements the Gnat.Server behaviour\n  subscription_topics: [\n    %{topic: \"rpc.MyApp.search\", queue_group: \"rpc.MyApp.search\"},\n    %{topic: \"rpc.MyApp.create\", queue_group: \"rpc.MyApp.create\"},\n  ],\n}\nworker(Gnat.ConsumerSupervisor, [consumer_supervisor_settings, [name: :rpc_consumer]], shutdown: 30_000)\n```\n\nThe second argument is a keyword list that gets used as the GenServer options so you can pass a name that you want to register for the consumer process if you like. The `:consuming_function` specifies which module and function to call when messages arrive. The function will be called with a single argument which is a `t:Gnat.message/0` just like you get when you call `Gnat.sub/4` directly.\n\nYou can have a single consumer that subscribes to multiple topics or multiple consumers that subscribe to different topics and call different consuming functions. It is recommended that your `ConsumerSupervisor`s are present later in your supervision tree than your `ConnectionSupervisor`. That way during a shutdown the `ConsumerSupervisor` can attempt a graceful shutdown of the consumer before shutting down the connection.\n\nIf you want this consumer supervisor to host a NATS service, then you can specify a module that\nimplements the `Gnat.Services.Server` behavior. You'll need to specify the `service_definition` field in the consumer\nsupervisor settings and conforms to the `Gnat.Services.Server.service_configuration` type. Here is an example of configuring\nthe consumer supervisor to manage a service:\n\n```\nconsumer_supervisor_settings = %{\n  connection_name: :name_of_supervised_connection,\n  module: MyApp.Service, # a module that implements the Gnat.Services.Server behaviour\n  service_definition: %{\n    name: \"exampleservice\",\n    description: \"This is an example service\",\n    version: \"0.1.0\",\n    endpoints: [\n      %{\n        name: \"add\",\n        group_name: \"calc\",\n      },\n      %{\n        name: \"sub\",\n        group_name: \"calc\"\n      }\n    ]\n  }\n}\nworker(Gnat.ConsumerSupervisor, [consumer_supervisor_settings, [name: :myservice_consumer]], shutdown: 30_000)\n```\n\nIt's also possible to pass a `%{consuming_function: {YourModule, :your_function}}` rather than a `:module` in your settings.\nIn that case no error handling or replying is taking care of for you, microservices cannot be used, and it will be up to your function to take whatever action you want with each message.","ref":"Gnat.ConsumerSupervisor.html"},{"type":"function","title":"Gnat.ConsumerSupervisor.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Gnat.ConsumerSupervisor.html#child_spec/1"},{"type":"function","title":"Gnat.ConsumerSupervisor.start_link/2","doc":"","ref":"Gnat.ConsumerSupervisor.html#start_link/2"},{"type":"module","title":"Gnat.Jetstream","doc":"Provides functions for interacting with a [NATS Jetstream](https://github.com/nats-io/jetstream)\nserver.","ref":"Gnat.Jetstream.html"},{"type":"function","title":"Gnat.Jetstream.ack/1","doc":"Sends `AckAck` acknowledgement to the server.\n\nAcknowledges a message was completely handled.","ref":"Gnat.Jetstream.html#ack/1"},{"type":"function","title":"Gnat.Jetstream.ack_next/2","doc":"Sends `AckNext` acknowledgement to the server.\n\nAcknowledges the message was handled and requests delivery of the next message to the reply\nsubject. Only applies to Pull-mode.","ref":"Gnat.Jetstream.html#ack_next/2"},{"type":"function","title":"Gnat.Jetstream.ack_progress/1","doc":"Sends `AckProgress` acknowledgement to the server.\n\nWhen sent before the `AckWait` period indicates that work is ongoing and the period should be\nextended by another equal to `AckWait`.","ref":"Gnat.Jetstream.html#ack_progress/1"},{"type":"function","title":"Gnat.Jetstream.ack_term/1","doc":"Sends `AckTerm` acknowledgement to the server.\n\nInstructs the server to stop redelivery of a message without acknowledging it as successfully\nprocessed.","ref":"Gnat.Jetstream.html#ack_term/1"},{"type":"function","title":"Gnat.Jetstream.nack/1","doc":"Sends `AckNak` acknowledgement to the server.\n\nSignals that the message will not be processed now and processing can move onto the next message.\nNAK'd message will be retried.","ref":"Gnat.Jetstream.html#nack/1"},{"type":"module","title":"Gnat.Jetstream.API.Consumer","doc":"A module representing a NATS JetStream Consumer.\n\nLearn more about consumers: https://docs.nats.io/nats-concepts/jetstream/consumers","ref":"Gnat.Jetstream.API.Consumer.html"},{"type":"module","title":"The Jetstream.API.Consumer struct - Gnat.Jetstream.API.Consumer","doc":"The struct's only mandatory field to set is the `:stream_name`. The rest will have\nthe NATS default values set.\n\nNote that consumers are ephemeral by default. Set the `:durable_name` to make it durable.\n\nConsumer struct fields explanation:\n\n* `:stream_name` - name of a stream the consumer is pointing at.\n* `:domain` - JetStream domain the stream is on.\n* `:ack_policy` - how the messages should be acknowledged. It has the following options:\n    - `:explicit` - the default policy. It means that each individual message must be acknowledged.\n      It is the only allowed option for pull consumers.\n    - `:none` - no need to ack messages, the server will assume ack on delivery.\n    - `:all` - only the last received message needs to be acked, all the previous messages received\n      are automatically acknowledged.\n* `:ack_wait` - time in nanoseconds that server will wait for an ack for any individual. If an ack\n   is not received in time, the message will be redelivered.\n* `:backoff` - list of durations that represents a retry timescale for NAK'd messages or those being\n   normally retried.\n* `:deliver_group` - when set, will only deliver messages to subscriptions matching that group.\n* `:deliver_policy` - specifies where in the stream it wants to start receiving messages. It has the\n   following options:\n     - `:all` - the default policy. The consumer will start receiving from the earliest available\n       message.\n     - `:last` - the consumer will start receiving messages with the last message added to the stream.\n     - `:new` - the consumer will only start receiving messages that were created after the customer\n       was created.\n     - `:by_start_sequence` - the consumer is required to specify `:opt_start_seq`, the sequence number\n       to start on. It will receive the closest available message moving forward in the sequence\n       should the message specified have been removed based on the stream limit policy.\n     - `:by_start_time` - the consumer will start with messages on or after this time. The consumer is\n       required to specify `:opt_start_time`, the time in the stream to start at.\n     - `:last_per_subject` - the consumer will start with the latest one for each filtered subject\n       currently  in the stream.\n* `:deliver_subject` - the subject to deliver observed messages. Not allowed for pull subscriptions.\n  A delivery subject is required for queue subscribing as it configures a subject that all the queue\n  consumers should listen on.\n* `:description` - a short description of the purpose of this customer.\n* `:durable_name` - the name of the consumer, which the server will track, allowing resuming consumption\n  where left off. By default, a consumer is ephemeral. To make the consumer durable, set the name.\n  See [naming](https://docs.nats.io/running-a-nats-service/nats_admin/jetstream_admin/naming).\n* `:filter_subject` - when consuming from a stream with a wildcard subject, this allows you to select\n  a subset of the full wildcard subject to receive messages from.\n* `:flow_control` - when set to true, an empty message with Status header 100 and a reply subject will\n  be sent. Consumers must reply to these messages to control the rate of message delivery.\n* `:headers_only` - delivers only the headers of messages in the stream and not the bodies. Additionally\n  adds the Nats-Msg-Size header to indicate the size of the removed payload.\n* `:idle_heartbeat` - if set, the server will regularly send a status message to the client while there\n  are  no new messages to send. This lets the client know that the JetStream service is still up and\n  running, even when there is no activity on the stream. The message status header will have a code of 100.\n  Unlike `:flow_control`, it will have no reply to address. It may have a description like\n  \"Idle Heartbeat\".\n* `:inactive_threshold` - duration that instructs the server to clean up ephemeral consumers that are\n  inactive for that long.\n* `:max_ack_pending` - it sets the maximum number of messages without an acknowledgement that can be\n  outstanding, once this limit is reached, message delivery will be suspended. It cannot be used with\n  `:ack_none` ack policy. This maximum number of pending acks applies for all the consumer's\n  subscriber processes. A value of -1 means there can be any number of pending acks (i.e. no flow\n  control).\n* `:max_batch` - the largest batch property that may be specified when doing a pull on a Pull consumer.\n* `:max_deliver` - the maximum number of times a specific message will be delivered. Applies to any\n  message that is re-sent due to ack policy.\n* `:max_expires` - the maximum expires value that may be set when doing a pull on a Pull consumer.\n* `:max_waiting` - the number of pulls that can be outstanding on a pull consumer, pulls received after\n  this is reached are ignored.\n* `:opt_start_seq` - use with `:deliver_policy` set to `:by_start_sequence`. It represents the sequence\n  number to start consuming on.\n* `:opt_start_time` - use with `:deliver_policy` set to `:by_start_time`. It represents the time to start\n  consuming at.\n* `:rate_limit_bps` - used to throttle the delivery of messages to the consumer, in bits per second.\n* `:replay_policy` - it applies when the `:deliver_policy` is set to `:all`, `:by_start_sequence` or\n  `:by_start_time`. It has the following options:\n     - `:instant` - the default policy. The messages will be pushed to the client as fast as possible.\n     - `:original` - the messages in the stream will be pushed to the client at the same rate that they\n       were originally received.\n* `:sample_freq` - Sets the percentage of acknowledgements that should be sampled for observability, 0-100.\n  This value is a binary and for example allows both `30` and `30%` as valid values.","ref":"Gnat.Jetstream.API.Consumer.html#module-the-jetstream-api-consumer-struct"},{"type":"type","title":"Gnat.Jetstream.API.Consumer.config/0","doc":"","ref":"Gnat.Jetstream.API.Consumer.html#t:config/0"},{"type":"type","title":"Gnat.Jetstream.API.Consumer.consumers/0","doc":"","ref":"Gnat.Jetstream.API.Consumer.html#t:consumers/0"},{"type":"function","title":"Gnat.Jetstream.API.Consumer.create/2","doc":"Creates a consumer. When consumer's `:durable_name` field is not set, the function\ncreates an ephemeral consumer. Otherwise, it creates a durable consumer.","ref":"Gnat.Jetstream.API.Consumer.html#create/2"},{"type":"function","title":"Examples - Gnat.Jetstream.API.Consumer.create/2","doc":"iex> {:ok, _response} = Gnat.Jetstream.API.Stream.create(:gnat, %Gnat.Jetstream.API.Stream{name: \"astream\", subjects: [\"subject\"]})\n    iex> {:ok, %{name: \"consumer\", stream_name: \"astream\"}} = Gnat.Jetstream.API.Consumer.create(:gnat, %Gnat.Jetstream.API.Consumer{durable_name: \"consumer\", stream_name: \"astream\"})\n\n    iex> {:ok, _response} = Gnat.Jetstream.API.Stream.create(:gnat, %Gnat.Jetstream.API.Stream{name: \"astream\", subjects: [\"subject\"]})\n    iex> {:error, %{\"description\" => \"consumer delivery policy is deliver by start sequence, but optional start sequence is not set\"}} = Gnat.Jetstream.API.Consumer.create(:gnat, %Gnat.Jetstream.API.Consumer{durable_name: \"consumer\", stream_name: \"astream\", deliver_policy: :by_start_sequence})","ref":"Gnat.Jetstream.API.Consumer.html#create/2-examples"},{"type":"function","title":"Gnat.Jetstream.API.Consumer.delete/4","doc":"Deletes a consumer.","ref":"Gnat.Jetstream.API.Consumer.html#delete/4"},{"type":"function","title":"Examples - Gnat.Jetstream.API.Consumer.delete/4","doc":"iex> {:ok, _response} = Gnat.Jetstream.API.Stream.create(:gnat, %Gnat.Jetstream.API.Stream{name: \"astream\", subjects: [\"subject\"]})\n    iex> {:ok, _response} = Gnat.Jetstream.API.Consumer.create(:gnat, %Gnat.Jetstream.API.Consumer{durable_name: \"consumer\", stream_name: \"astream\"})\n    iex> Gnat.Jetstream.API.Consumer.delete(:gnat, \"astream\", \"consumer\")\n    :ok\n\n    iex> {:error, %{\"code\" => 404, \"description\" => \"stream not found\"}} = Gnat.Jetstream.API.Consumer.delete(:gnat, \"wrong_stream\", \"consumer\")","ref":"Gnat.Jetstream.API.Consumer.html#delete/4-examples"},{"type":"type","title":"Gnat.Jetstream.API.Consumer.info/0","doc":"","ref":"Gnat.Jetstream.API.Consumer.html#t:info/0"},{"type":"function","title":"Gnat.Jetstream.API.Consumer.info/4","doc":"Information about the consumer.","ref":"Gnat.Jetstream.API.Consumer.html#info/4"},{"type":"function","title":"Examples - Gnat.Jetstream.API.Consumer.info/4","doc":"iex> {:ok, _response} = Gnat.Jetstream.API.Stream.create(:gnat, %Gnat.Jetstream.API.Stream{name: \"astream\", subjects: [\"subject\"]})\n    iex> {:ok, _response} = Gnat.Jetstream.API.Consumer.create(:gnat, %Gnat.Jetstream.API.Consumer{durable_name: \"consumer\", stream_name: \"astream\"})\n    iex> {:ok, %{created: _}} = Gnat.Jetstream.API.Consumer.info(:gnat, \"astream\", \"consumer\")\n\n    iex>  {:error, %{\"code\" => 404, \"description\" => \"stream not found\"}} = Gnat.Jetstream.API.Consumer.info(:gnat, \"wrong_stream\", \"consumer\")","ref":"Gnat.Jetstream.API.Consumer.html#info/4-examples"},{"type":"function","title":"Gnat.Jetstream.API.Consumer.list/3","doc":"Paged list of known consumers, including their current info.","ref":"Gnat.Jetstream.API.Consumer.html#list/3"},{"type":"function","title":"Examples - Gnat.Jetstream.API.Consumer.list/3","doc":"iex> {:ok, _response} =  Gnat.Jetstream.API.Stream.create(:gnat, %Gnat.Jetstream.API.Stream{name: \"astream\", subjects: [\"subject\"]})\n    iex> {:ok, %{consumers: _, limit: 1024, offset: 0, total: _}} = Gnat.Jetstream.API.Consumer.list(:gnat, \"astream\")\n\n    iex> {:error, %{\"code\" => 404, \"description\" => \"stream not found\"}} = Gnat.Jetstream.API.Consumer.list(:gnat, \"wrong_stream\")","ref":"Gnat.Jetstream.API.Consumer.html#list/3-examples"},{"type":"function","title":"Gnat.Jetstream.API.Consumer.request_next_message/6","doc":"Requests a next message from a stream to be consumed. The response (consumed message)will be sent\non the subject given as the `reply_to` parameter.","ref":"Gnat.Jetstream.API.Consumer.html#request_next_message/6"},{"type":"function","title":"Options - Gnat.Jetstream.API.Consumer.request_next_message/6","doc":"* `batch` - How many messages to receive. Messages will be sent to the `reply_to` subject\n  separately. Defaults to 1.\n\n* `expires` - Time in nanoseconds the request will be kept in the server. Once this time passes\n  a message with empty body and topic set to `reply_to` subject is sent. Useful when polling\n  the server frequently and not wanting the pull requests to accumulate. By default, the pull\n  request stays in the server until a message comes.\n\n* `no_wait` - Boolean value which indicates whether the pull request should be accumulated on\n  the server. When set to true and no message is present to be consumed, a message with empty\n  body and topic value set to `reply_to` is sent. Defaults to false.","ref":"Gnat.Jetstream.API.Consumer.html#request_next_message/6-options"},{"type":"function","title":"Example - Gnat.Jetstream.API.Consumer.request_next_message/6","doc":"iex> {:ok, _response} = Gnat.Jetstream.API.Stream.create(:gnat, %Gnat.Jetstream.API.Stream{name: \"astream\", subjects: [\"subject\"]})\n    iex> {:ok, _response} = Gnat.Jetstream.API.Consumer.create(:gnat, %Gnat.Jetstream.API.Consumer{durable_name: \"consumer\", stream_name: \"astream\"})\n    iex> {:ok, _sid} = Gnat.sub(:gnat, self(), \"reply_subject\")\n    iex> :ok = Gnat.Jetstream.API.Consumer.request_next_message(:gnat, \"astream\", \"consumer\", \"reply_subject\")\n    iex> :ok = Gnat.pub(:gnat, \"subject\", \"message1\")\n    iex> assert_receive {:msg, %{body: \"message1\", topic: \"subject\"}}","ref":"Gnat.Jetstream.API.Consumer.html#request_next_message/6-example"},{"type":"type","title":"Gnat.Jetstream.API.Consumer.t/0","doc":"","ref":"Gnat.Jetstream.API.Consumer.html#t:t/0"},{"type":"module","title":"Gnat.Jetstream.API.KV","doc":"API for interacting with the Key/Value store functionality in Nats Jetstream.\n\nLearn about the Key/Value store: https://docs.nats.io/nats-concepts/jetstream/key-value-store","ref":"Gnat.Jetstream.API.KV.html"},{"type":"type","title":"Gnat.Jetstream.API.KV.bucket_options/0","doc":"","ref":"Gnat.Jetstream.API.KV.html#t:bucket_options/0"},{"type":"function","title":"Gnat.Jetstream.API.KV.contents/3","doc":"Get all the non-deleted key-value pairs for a Bucket","ref":"Gnat.Jetstream.API.KV.html#contents/3"},{"type":"function","title":"Examples - Gnat.Jetstream.API.KV.contents/3","doc":"iex> {:ok, %{\"key1\" => \"value1\"}} = Jetstream.API.KV.contents(:gnat, \"my_bucket\")","ref":"Gnat.Jetstream.API.KV.html#contents/3-examples"},{"type":"function","title":"Gnat.Jetstream.API.KV.create_bucket/3","doc":"Create a new Key/Value bucket. Can include the following options\n\n* `:history` - How many historic values to keep per key (defaults to 1, max of 64)\n* `:ttl` - How long to keep values for (in nanoseconds)\n* `:max_bucket_size` - The max number of bytes the bucket can hold\n* `:max_value_size` - The max number of bytes a value may be\n* `:description` - A description for the bucket\n* `:replicas` - How many replicas of the data to store\n* `:storage` - Storage backend to use (:file, :memory)\n* `:placement` - A map with :cluster (required) and :tags (optional)","ref":"Gnat.Jetstream.API.KV.html#create_bucket/3"},{"type":"function","title":"Examples - Gnat.Jetstream.API.KV.create_bucket/3","doc":"iex> {:ok, info} = Jetstream.API.KV.create_bucket(:gnat, \"my_bucket\")","ref":"Gnat.Jetstream.API.KV.html#create_bucket/3-examples"},{"type":"function","title":"Gnat.Jetstream.API.KV.create_key/5","doc":"Create a Key in a Key/Value Bucket","ref":"Gnat.Jetstream.API.KV.html#create_key/5"},{"type":"function","title":"Options - Gnat.Jetstream.API.KV.create_key/5","doc":"* `:timeout` - receive timeout for the request","ref":"Gnat.Jetstream.API.KV.html#create_key/5-options"},{"type":"function","title":"Examples - Gnat.Jetstream.API.KV.create_key/5","doc":"iex> :ok = Jetstream.API.KV.create_key(:gnat, \"my_bucket\", \"my_key\", \"my_value\")","ref":"Gnat.Jetstream.API.KV.html#create_key/5-examples"},{"type":"function","title":"Gnat.Jetstream.API.KV.delete_bucket/2","doc":"Delete a Key/Value bucket","ref":"Gnat.Jetstream.API.KV.html#delete_bucket/2"},{"type":"function","title":"Examples - Gnat.Jetstream.API.KV.delete_bucket/2","doc":"iex> :ok = Jetstream.API.KV.delete_bucket(:gnat, \"my_bucket\")","ref":"Gnat.Jetstream.API.KV.html#delete_bucket/2-examples"},{"type":"function","title":"Gnat.Jetstream.API.KV.delete_key/4","doc":"Delete a Key from a K/V Bucket","ref":"Gnat.Jetstream.API.KV.html#delete_key/4"},{"type":"function","title":"Examples - Gnat.Jetstream.API.KV.delete_key/4","doc":"iex> :ok = Jetstream.API.KV.delete_key(:gnat, \"my_bucket\", \"my_key\")","ref":"Gnat.Jetstream.API.KV.html#delete_key/4-examples"},{"type":"function","title":"Gnat.Jetstream.API.KV.get_value/3","doc":"Get the value for a key in a particular K/V bucket","ref":"Gnat.Jetstream.API.KV.html#get_value/3"},{"type":"function","title":"Examples - Gnat.Jetstream.API.KV.get_value/3","doc":"iex> \"my_value\" = Jetstream.API.KV.get_value(:gnat, \"my_bucket\", \"my_key\")","ref":"Gnat.Jetstream.API.KV.html#get_value/3-examples"},{"type":"function","title":"Gnat.Jetstream.API.KV.info/3","doc":"Information about the state of the bucket's Stream.","ref":"Gnat.Jetstream.API.KV.html#info/3"},{"type":"function","title":"Opts - Gnat.Jetstream.API.KV.info/3","doc":"* `:domain` - (default `nil`) the domain of the bucket","ref":"Gnat.Jetstream.API.KV.html#info/3-opts"},{"type":"function","title":"Gnat.Jetstream.API.KV.is_kv_bucket_stream?/1","doc":"","ref":"Gnat.Jetstream.API.KV.html#is_kv_bucket_stream?/1"},{"type":"function","title":"Gnat.Jetstream.API.KV.kv_bucket_stream?/1","doc":"Returns true if the provided stream is a KV bucket, false otherwise","ref":"Gnat.Jetstream.API.KV.html#kv_bucket_stream?/1"},{"type":"function","title":"Parameters - Gnat.Jetstream.API.KV.kv_bucket_stream?/1","doc":"* `stream_name` - the stream name to test","ref":"Gnat.Jetstream.API.KV.html#kv_bucket_stream?/1-parameters"},{"type":"function","title":"Gnat.Jetstream.API.KV.list_buckets/1","doc":"Returns a list of all the buckets in the KV","ref":"Gnat.Jetstream.API.KV.html#list_buckets/1"},{"type":"function","title":"Gnat.Jetstream.API.KV.purge_key/4","doc":"Purge a Key from a K/V bucket. This will remove any revision history the key had","ref":"Gnat.Jetstream.API.KV.html#purge_key/4"},{"type":"function","title":"Examples - Gnat.Jetstream.API.KV.purge_key/4","doc":"iex> :ok = Jetstream.API.KV.purge_key(:gnat, \"my_bucket\", \"my_key\")","ref":"Gnat.Jetstream.API.KV.html#purge_key/4-examples"},{"type":"function","title":"Gnat.Jetstream.API.KV.put_value/5","doc":"Put a value into a Key in a K/V Bucket","ref":"Gnat.Jetstream.API.KV.html#put_value/5"},{"type":"function","title":"Examples - Gnat.Jetstream.API.KV.put_value/5","doc":"iex> :ok = Jetstream.API.KV.put_value(:gnat, \"my_bucket\", \"my_key\", \"my_value\")","ref":"Gnat.Jetstream.API.KV.html#put_value/5-examples"},{"type":"function","title":"Gnat.Jetstream.API.KV.unwatch/1","doc":"Stops a previously running monitor. This will unsubscribe from the key changes and remove the\nephemeral consumer","ref":"Gnat.Jetstream.API.KV.html#unwatch/1"},{"type":"function","title":"Examples - Gnat.Jetstream.API.KV.unwatch/1","doc":"iex> :ok = Jetstream.API.KV.unwatch(pid)","ref":"Gnat.Jetstream.API.KV.html#unwatch/1-examples"},{"type":"function","title":"Gnat.Jetstream.API.KV.watch/3","doc":"Starts a monitor for key changes in a given bucket. Supply a handler that will receive\nkey change notifications.","ref":"Gnat.Jetstream.API.KV.html#watch/3"},{"type":"function","title":"Examples - Gnat.Jetstream.API.KV.watch/3","doc":"iex> {:ok, _pid} = Jetstream.API.KV.watch(:gnat, \"my_bucket\", fn action, key, value ->\n    ...>  IO.puts(\"#{action} taken on #{key}\")\n    ...> end)","ref":"Gnat.Jetstream.API.KV.html#watch/3-examples"},{"type":"module","title":"Gnat.Jetstream.API.KV.Watcher","doc":"The watcher server establishes a subscription to the changes that occur to a given key-value bucket. The\nconsumer-supplied handler function will be sent an indicator as to whether the change is a delete or an add,\nas well as the key being changed and the value (if it was added).\n\nEnsure that you call `stop` with a watcher pid when you no longer need to be notified about key changes","ref":"Gnat.Jetstream.API.KV.Watcher.html"},{"type":"function","title":"Gnat.Jetstream.API.KV.Watcher.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Gnat.Jetstream.API.KV.Watcher.html#child_spec/1"},{"type":"function","title":"Gnat.Jetstream.API.KV.Watcher.init/1","doc":"Callback implementation for  GenServer.init/1 .","ref":"Gnat.Jetstream.API.KV.Watcher.html#init/1"},{"type":"type","title":"Gnat.Jetstream.API.KV.Watcher.keywatch_handler/0","doc":"","ref":"Gnat.Jetstream.API.KV.Watcher.html#t:keywatch_handler/0"},{"type":"function","title":"Gnat.Jetstream.API.KV.Watcher.start_link/1","doc":"","ref":"Gnat.Jetstream.API.KV.Watcher.html#start_link/1"},{"type":"function","title":"Gnat.Jetstream.API.KV.Watcher.stop/1","doc":"","ref":"Gnat.Jetstream.API.KV.Watcher.html#stop/1"},{"type":"type","title":"Gnat.Jetstream.API.KV.Watcher.watcher_options/0","doc":"","ref":"Gnat.Jetstream.API.KV.Watcher.html#t:watcher_options/0"},{"type":"module","title":"Gnat.Jetstream.API.Message.Metadata","doc":"","ref":"Gnat.Jetstream.API.Message.Metadata.html"},{"type":"type","title":"Gnat.Jetstream.API.Message.Metadata.t/0","doc":"","ref":"Gnat.Jetstream.API.Message.Metadata.html#t:t/0"},{"type":"module","title":"Gnat.Jetstream.API.Object","doc":"API for interacting with the JetStream Object Store\n\nLearn more about Object Store: https://docs.nats.io/nats-concepts/jetstream/obj_store","ref":"Gnat.Jetstream.API.Object.html"},{"type":"type","title":"Gnat.Jetstream.API.Object.bucket_opt/0","doc":"","ref":"Gnat.Jetstream.API.Object.html#t:bucket_opt/0"},{"type":"function","title":"Gnat.Jetstream.API.Object.create_bucket/3","doc":"","ref":"Gnat.Jetstream.API.Object.html#create_bucket/3"},{"type":"function","title":"Gnat.Jetstream.API.Object.delete/3","doc":"","ref":"Gnat.Jetstream.API.Object.html#delete/3"},{"type":"function","title":"Gnat.Jetstream.API.Object.delete_bucket/2","doc":"","ref":"Gnat.Jetstream.API.Object.html#delete_bucket/2"},{"type":"function","title":"Gnat.Jetstream.API.Object.get/4","doc":"","ref":"Gnat.Jetstream.API.Object.html#get/4"},{"type":"function","title":"Gnat.Jetstream.API.Object.info/3","doc":"","ref":"Gnat.Jetstream.API.Object.html#info/3"},{"type":"function","title":"Gnat.Jetstream.API.Object.is_object_bucket_stream?/1","doc":"Returns true if the provided stream is an Object bucket, false otherwise","ref":"Gnat.Jetstream.API.Object.html#is_object_bucket_stream?/1"},{"type":"function","title":"Parameters - Gnat.Jetstream.API.Object.is_object_bucket_stream?/1","doc":"* `stream_name` - the stream name to test","ref":"Gnat.Jetstream.API.Object.html#is_object_bucket_stream?/1-parameters"},{"type":"function","title":"Gnat.Jetstream.API.Object.list/3","doc":"","ref":"Gnat.Jetstream.API.Object.html#list/3"},{"type":"function","title":"Gnat.Jetstream.API.Object.list_buckets/1","doc":"Returns a list of all Object buckets","ref":"Gnat.Jetstream.API.Object.html#list_buckets/1"},{"type":"type","title":"Gnat.Jetstream.API.Object.list_option/0","doc":"","ref":"Gnat.Jetstream.API.Object.html#t:list_option/0"},{"type":"function","title":"Gnat.Jetstream.API.Object.put/4","doc":"","ref":"Gnat.Jetstream.API.Object.html#put/4"},{"type":"module","title":"Gnat.Jetstream.API.Object.Meta","doc":"","ref":"Gnat.Jetstream.API.Object.Meta.html"},{"type":"type","title":"Gnat.Jetstream.API.Object.Meta.t/0","doc":"","ref":"Gnat.Jetstream.API.Object.Meta.html#t:t/0"},{"type":"module","title":"Gnat.Jetstream.API.Stream","doc":"A module representing a NATS JetStream Stream.\n\nLearn more about Streams: https://docs.nats.io/nats-concepts/jetstream/streams","ref":"Gnat.Jetstream.API.Stream.html"},{"type":"module","title":"The Jetstream.API.Stream struct - Gnat.Jetstream.API.Stream","doc":"The struct's mandatory fields are `:name` and `:subjects`. The rest will have the NATS\ndefault values set.\n\nStream struct fields explanation:\n\n* `:allow_direct` - Allow higher performance, direct access to get individual messages. E.g. KeyValue\n* `:allow_rollup_hdrs` - allows the use of the Nats-Rollup header to replace all contents of a stream,\n  or subject in a stream, with a single new message.\n* `:deny_delete` - restricts the ability to delete messages from a stream via the API. Cannot be changed\n  once set to true.\n* `:deny_purge` - restricts the ability to purge messages from a stream via the API. Cannot be change\n  once set to true.\n* `:description` - a short description of the purpose of this stream.\n* `:discard` - determines what happens when a Stream reaches its limits. It has the following options:\n   - `:old` - the default option. Old messages are deleted.\n   - `:new` - refuses new messages.\n* `:discard_new_per_subject` - - allows to enable discarding new messages per subject when limits are reached.\n  Requires `discard: :new` and the `:max_msgs_per_subject` to be configured.\n* `:domain` - JetStream domain, mainly used for leaf nodes.\n   See [JetStream on Leaf Nodes](https://docs.nats.io/running-a-nats-service/configuration/leafnodes/jetstream_leafnodes).\n* `:duplicate_window` - the window within which to track duplicate messages, expressed in nanoseconds.\n* `:max_age` - maximum age of any message in the Stream, expressed in nanoseconds.\n* `:max_bytes` - how many bytes the Stream may contain. Adheres to `:discard`, removing oldest or\n  refusing new messages if the Stream exceeds this size.\n* `:max_consumers` - how many Consumers can be defined for a given Stream, -1 for unlimited.\n* `:max_msg_size` - the largest message that will be accepted by the Stream.\n* `:max_msgs_per_subject` - For wildcard streams ensure that for every unique subject this many messages are kept - a per subject retention limit.\n  Only available on nats-server versions greater than 2.3.0\n* `:max_msgs` - how many messages may be in a Stream. Adheres to `:discard`, removing oldest or refusing\n  new messages if the Stream exceeds this number of messages\n* `:mirror` - maintains a 1:1 mirror of another stream with name matching this property.  When a mirror\n  is configured subjects and sources must be empty.\n* `:mirror_direct` - Allow higher performance and unified direct access for mirrors as well.\n* `:name` - a name for the Stream.\n  See [naming](https://docs.nats.io/running-a-nats-service/nats_admin/jetstream_admin/naming).\n* `:no_ack` - disables acknowledging messages that are received by the Stream.\n* `:num_replicas` - how many replicas to keep for each message.\n* `:placement` - placement directives to consider when placing replicas of this stream, random placement\n  when unset. It has the following properties:\n   - `:cluster` - the desired cluster name to place the stream.\n   - `:tags` - tags required on servers hosting this stream.\n* `:retention` - how messages are retained in the Stream. Once this is exceeded, old messages are removed.\n  It has the following options:\n   - `:limits` - the default policy.\n   - `:interest`\n   - `:workqueue`\n* `:sealed` - sealed streams do not allow messages to be deleted via limits or API, sealed streams can not\n  be unsealed via configuration update. Can only be set on already created streams via the Update API.\n* `:sources` - list of stream names to replicate into this stream.\n* `:storage` - the type of storage backend. Available options:\n   - `:file`\n   - `:memory`\n* `:compression` - If file-based and a compression algorithm is specified, the stream data will be compressed on disk.\n  Valid options are \"none\" for no compression or \"s2\" for Snappy compression.\n* `:subjects` - a list of subjects to consume, supports wildcards.\n* `:template_owner` - when the Stream is managed by a Stream Template this identifies the template that\n  manages the Stream.","ref":"Gnat.Jetstream.API.Stream.html#module-the-jetstream-api-stream-struct"},{"type":"function","title":"Gnat.Jetstream.API.Stream.create/2","doc":"Creates a new Stream.","ref":"Gnat.Jetstream.API.Stream.html#create/2"},{"type":"function","title":"Examples - Gnat.Jetstream.API.Stream.create/2","doc":"iex> {:ok, %{created: _}} = Gnat.Jetstream.API.Stream.create(:gnat, %Gnat.Jetstream.API.Stream{name: \"anewstream\", subjects: [\"anewsubject\"]})","ref":"Gnat.Jetstream.API.Stream.html#create/2-examples"},{"type":"function","title":"Gnat.Jetstream.API.Stream.delete/3","doc":"Deletes a Stream and all its data.","ref":"Gnat.Jetstream.API.Stream.html#delete/3"},{"type":"function","title":"Examples - Gnat.Jetstream.API.Stream.delete/3","doc":"iex> Gnat.Jetstream.API.Stream.create(:gnat, %Gnat.Jetstream.API.Stream{name: \"delstream\", subjects: [\"delsubject\"]})\n    iex> Gnat.Jetstream.API.Stream.delete(:gnat, \"delstream\")\n    :ok\n\n    iex> {:error, %{\"code\" => 404, \"description\" => \"stream not found\"}} = Gnat.Jetstream.API.Stream.delete(:gnat, \"wrong_stream\")","ref":"Gnat.Jetstream.API.Stream.html#delete/3-examples"},{"type":"function","title":"Gnat.Jetstream.API.Stream.get_message/4","doc":"Get a message from the stream either by \"stream sequence number\" or the \"last message for a given subject\"","ref":"Gnat.Jetstream.API.Stream.html#get_message/4"},{"type":"type","title":"Gnat.Jetstream.API.Stream.info/0","doc":"","ref":"Gnat.Jetstream.API.Stream.html#t:info/0"},{"type":"function","title":"Gnat.Jetstream.API.Stream.info/3","doc":"Information about config and state of a Stream.","ref":"Gnat.Jetstream.API.Stream.html#info/3"},{"type":"function","title":"Examples - Gnat.Jetstream.API.Stream.info/3","doc":"iex> {:ok, _} = Gnat.Jetstream.API.Stream.create(:gnat, %Gnat.Jetstream.API.Stream{name: \"infostream\", subjects: [\"infosubject\"]})\n    iex> {:ok, %{created: _}} = Gnat.Jetstream.API.Stream.info(:gnat, \"infostream\")\n\n    iex> {:error, %{\"code\" => 404, \"description\" => \"stream not found\"}} = Gnat.Jetstream.API.Stream.info(:gnat, \"wrong_stream\")","ref":"Gnat.Jetstream.API.Stream.html#info/3-examples"},{"type":"function","title":"Gnat.Jetstream.API.Stream.list/2","doc":"Paged list of known Streams including all their current information.","ref":"Gnat.Jetstream.API.Stream.html#list/2"},{"type":"function","title":"Options - Gnat.Jetstream.API.Stream.list/2","doc":"* `:offset` - Number of records to skip\n* `:subject` - A subject the `Stream` must collect to appear in the list.","ref":"Gnat.Jetstream.API.Stream.html#list/2-options"},{"type":"function","title":"Examples - Gnat.Jetstream.API.Stream.list/2","doc":"iex> {:ok, %{total: _, offset: 0, limit: 1024, streams: _}} = Gnat.Jetstream.API.Stream.list(:gnat)","ref":"Gnat.Jetstream.API.Stream.html#list/2-examples"},{"type":"type","title":"Gnat.Jetstream.API.Stream.message_access_method/0","doc":"* `seq` - Stream sequence number of the message to retrieve, cannot be combined with `last_by_subj`\n* `last_by_subj` - Retrieves the last message for a given subject, cannot be combined with `seq`","ref":"Gnat.Jetstream.API.Stream.html#t:message_access_method/0"},{"type":"type","title":"Gnat.Jetstream.API.Stream.message_response/0","doc":"* `data` - The decoded message payload\n* `subject` - The subject the message was originally received on\n* `time` - The time the message was received\n* `seq` - The sequence number of the message in the Stream\n* `hdrs` - The decoded headers for the message","ref":"Gnat.Jetstream.API.Stream.html#t:message_response/0"},{"type":"type","title":"Gnat.Jetstream.API.Stream.method/0","doc":"","ref":"Gnat.Jetstream.API.Stream.html#t:method/0"},{"type":"type","title":"Gnat.Jetstream.API.Stream.nanoseconds/0","doc":"","ref":"Gnat.Jetstream.API.Stream.html#t:nanoseconds/0"},{"type":"type","title":"Gnat.Jetstream.API.Stream.placement/0","doc":"","ref":"Gnat.Jetstream.API.Stream.html#t:placement/0"},{"type":"function","title":"Gnat.Jetstream.API.Stream.purge/3","doc":"Purges all of data in the stream but doesn't delete the stream.","ref":"Gnat.Jetstream.API.Stream.html#purge/3"},{"type":"function","title":"Examples - Gnat.Jetstream.API.Stream.purge/3","doc":"iex> Gnat.Jetstream.API.Stream.create(:gnat, %Gnat.Jetstream.API.Stream{name: \"purgestream\", subjects: [\"purgesubject\"]})\n    iex> Gnat.Jetstream.API.Stream.purge(:gnat, \"purgestream\")\n    :ok\n\n    iex> {:error, %{\"code\" => 404, \"description\" => \"stream not found\"}} = Gnat.Jetstream.API.Stream.purge(:gnat, \"wrong_stream\")","ref":"Gnat.Jetstream.API.Stream.html#purge/3-examples"},{"type":"function","title":"Gnat.Jetstream.API.Stream.purge/4","doc":"Purges some of the messages in a stream according to the supplied filter","ref":"Gnat.Jetstream.API.Stream.html#purge/4"},{"type":"function","title":"Examples - Gnat.Jetstream.API.Stream.purge/4","doc":"iex> Gnat.Jetstream.API.Stream.create(:gnat, %Gnat.Jetstream.API.Stream{name: \"pstream\", subjects: [\"psub1\", \"psub2\"]})\n    iex> Gnat.Jetstream.API.Stream.purge(:gnat, \"pstream\", nil, %{filter: \"psub1\"})\n    :ok","ref":"Gnat.Jetstream.API.Stream.html#purge/4-examples"},{"type":"type","title":"Gnat.Jetstream.API.Stream.response_error/0","doc":"* `code` - HTTP like error code in the 300 to 500 range\n* `description` - A human friendly description of the error\n* `err_code` - The NATS error code unique to each kind of error","ref":"Gnat.Jetstream.API.Stream.html#t:response_error/0"},{"type":"type","title":"Gnat.Jetstream.API.Stream.source/0","doc":"Stream source fields explained:\n\n* `:name` - stream name.\n* `:opt_start_seq` - sequence to start replicating from.\n* `:opt_start_time` - timestamp to start replicating from.\n* `:filter_subject` - replicate only a subset of messages based on filter.\n* `:external` - configuration referencing a stream source in another account or JetStream domain.\n  It has the following parameters:\n   - `:api` - the subject prefix that imports other account/domain `$JS.API.CONSUMER.>` subjects\n   - `:deliver` - the delivery subject to use for push consumer","ref":"Gnat.Jetstream.API.Stream.html#t:source/0"},{"type":"type","title":"Gnat.Jetstream.API.Stream.source_info/0","doc":"","ref":"Gnat.Jetstream.API.Stream.html#t:source_info/0"},{"type":"type","title":"Gnat.Jetstream.API.Stream.state/0","doc":"","ref":"Gnat.Jetstream.API.Stream.html#t:state/0"},{"type":"type","title":"Gnat.Jetstream.API.Stream.streams/0","doc":"","ref":"Gnat.Jetstream.API.Stream.html#t:streams/0"},{"type":"type","title":"Gnat.Jetstream.API.Stream.t/0","doc":"","ref":"Gnat.Jetstream.API.Stream.html#t:t/0"},{"type":"function","title":"Gnat.Jetstream.API.Stream.update/2","doc":"Updates a Stream.","ref":"Gnat.Jetstream.API.Stream.html#update/2"},{"type":"function","title":"Examples - Gnat.Jetstream.API.Stream.update/2","doc":"iex> {:ok, %{created: _}} = Gnat.Jetstream.API.Stream.create(:gnat, %Gnat.Jetstream.API.Stream{name: \"update_test_stream\", subjects: [\"update_subject\"]})\n    iex> {:ok, _} = Gnat.Jetstream.API.Stream.update(:gnat, %Gnat.Jetstream.API.Stream{name: \"update_test_stream\", subjects: [\"update_subject\", \"new.update_subject\"]})","ref":"Gnat.Jetstream.API.Stream.html#update/2-examples"},{"type":"behaviour","title":"Gnat.Jetstream.PullConsumer","doc":"A behaviour which provides the NATS JetStream Pull Consumer functionalities.\n\nWhen a Consumer is pull-based, it means that the messages will be delivered when the server\nis asked for them.","ref":"Gnat.Jetstream.PullConsumer.html"},{"type":"behaviour","title":"Example - Gnat.Jetstream.PullConsumer","doc":"Declare a module which uses `Gnat.Jetstream.PullConsumer` and implements `c:init/1` and\n`c:handle_message/2` callbacks.\n\n    defmodule MyApp.PullConsumer do\n      use Gnat.Jetstream.PullConsumer\n\n      def start_link(arg) do\n        Jetstream.PullConsumer.start_link(__MODULE__, arg)\n      end\n\n      @impl true\n      def init(_arg) do\n        {:ok, nil,\n          connection_name: :gnat,\n          stream_name: \"TEST_STREAM\",\n          consumer_name: \"TEST_CONSUMER\"}\n      end\n\n      @impl true\n      def handle_message(message, state) do\n        # Do some processing with the message.\n        {:ack, state}\n      end\n    end\n\nYou can then place your Pull Consumer in a supervision tree. Remember that you need to have the\n`Gnat.ConnectionSupervisor` set up.\n\n    defmodule MyApp.Application do\n      use Application\n\n      @impl true\n      def start(_type, _args) do\n        children = [\n          # Create NATS connection\n          {Gnat.ConnectionSupervisor, ...},\n          # Start NATS Jetstream Pull Consumer\n          MyApp.PullConsumer,\n        ]\n        opts = [strategy: :one_for_one]\n        Supervisor.start_link(children, opts)\n      end\n    end","ref":"Gnat.Jetstream.PullConsumer.html#module-example"},{"type":"behaviour","title":"Connection Options - Gnat.Jetstream.PullConsumer","doc":"In order to establish consumer connection with NATS, you need to pass several connection options\nvia keyword list in third element of a tuple returned from `c:init/1` callback.\n\nFollowing options **must** be provided. Omitting this options will cause the process to raise\nerrors upon initialization:\n\n* `:connection_name` - Gnat connection or `Gnat.ConnectionSupervisor` name/PID.\n* `:stream_name` - name of an existing string the consumer will consume messages from.\n* `:consumer_name` - name of an existing consumer pointing at the stream.\n\nYou can also pass the optional ones:\n\n* `:connection_retry_timeout` - a duration in milliseconds after which the PullConsumer which\n  failed to establish NATS connection retries, defaults to `1000`\n* `:connection_retries` - a number of attempts the PullConsumer will make to establish the NATS\n  connection. When this value is exceeded, the pull consumer stops with the `:timeout` reason,\n  defaults to `10`\n* `:inbox_prefix` - allows the default `_INBOX.` prefix to be customized. Should end with a dot.\n* `:domain` - use a JetStream domain, this is mostly used on leaf nodes.","ref":"Gnat.Jetstream.PullConsumer.html#module-connection-options"},{"type":"behaviour","title":"Dynamic Connection Options - Gnat.Jetstream.PullConsumer","doc":"It is possible that you have to determine some of the options dynamically depending on pull\nconsumer's init argument. To do so, it is recommended to derive these options values from some\ninit argument:\n\n    defmodule MyApp.PullConsumer do\n      use Gnat.Jetstream.PullConsumer\n\n      def start_link() do\n        Gnat.Jetstream.PullConsumer.start_link(__MODULE__, %{counter: counter})\n      end\n\n      @impl true\n      def init(%{counter: counter}) do\n        {:ok, nil,\n          connection_name: :gnat,\n          stream_name: \"TEST_STREAM_#{counter}\",\n          consumer_name: \"TEST_CONSUMER_#{counter}\"}\n      end\n\n      ...\n    end","ref":"Gnat.Jetstream.PullConsumer.html#module-dynamic-connection-options"},{"type":"behaviour","title":"How to supervise - Gnat.Jetstream.PullConsumer","doc":"A `PullConsumer` is most commonly started under a supervision tree. When we invoke\n`use Gnat.Jetstream.PullConsumer`, it automatically defines a `child_spec/1` function that allows us\nto start the pull consumer directly under a supervisor. To start a pull consumer under\na supervisor with an initial argument of :example, one may do:\n\n    children = [\n      {MyPullConsumer, :example}\n    ]\n    Supervisor.start_link(children, strategy: :one_for_all)\n\nWhile one could also simply pass the `MyPullConsumer` as a child to the supervisor, such as:\n\n    children = [\n      MyPullConsumer # Same as {MyPullConsumer, []}\n    ]\n    Supervisor.start_link(children, strategy: :one_for_all)\n\nA common approach is to use a keyword list, which allows setting init argument and server options,\nfor example:\n\n    def start_link(opts) do\n      {initial_state, opts} = Keyword.pop(opts, :initial_state, nil)\n      Gnat.Jetstream.PullConsumer.start_link(__MODULE__, initial_state, opts)\n    end\n\nand then you can use `MyPullConsumer`, `{MyPullConsumer, name: :my_consumer}` or even\n`{MyPullConsumer, initial_state: :example, name: :my_consumer}` as a child specification.\n\n`use Gnat.Jetstream.PullConsumer` also accepts a list of options which configures the child\nspecification and therefore how it runs under a supervisor. The generated `child_spec/1` can be\ncustomized with the following options:\n\n  * `:id` - the child specification identifier, defaults to the current module\n  * `:restart` - when the child should be restarted, defaults to `:permanent`\n  * `:shutdown` - how to shut down the child, either immediately or by giving it time to shut down\n\nFor example:\n\n    use Gnat.Jetstream.PullConsumer, restart: :transient, shutdown: 10_000\n\nSee the \"Child specification\" section in the `Supervisor` module for more detailed information.\nThe `@doc` annotation immediately preceding `use Jetstream.PullConsumer` will be attached to\nthe generated `child_spec/1` function.","ref":"Gnat.Jetstream.PullConsumer.html#module-how-to-supervise"},{"type":"behaviour","title":"Name registration - Gnat.Jetstream.PullConsumer","doc":"A pull consumer is bound to the same name registration rules as GenServers.\nRead more about it in the `GenServer` documentation.","ref":"Gnat.Jetstream.PullConsumer.html#module-name-registration"},{"type":"function","title":"Gnat.Jetstream.PullConsumer.close/1","doc":"Closes the pull consumer and stops underlying process.","ref":"Gnat.Jetstream.PullConsumer.html#close/1"},{"type":"function","title":"Example - Gnat.Jetstream.PullConsumer.close/1","doc":"{:ok, consumer} =\n      PullConsumer.start_link(ExamplePullConsumer,\n        connection_name: :gnat,\n        stream_name: \"TEST_STREAM\",\n        consumer_name: \"TEST_CONSUMER\"\n      )\n\n    :ok = PullConsumer.close(consumer)","ref":"Gnat.Jetstream.PullConsumer.html#close/1-example"},{"type":"type","title":"Gnat.Jetstream.PullConsumer.connection_option/0","doc":"Connection option values used to connect the consumer to NATS server.","ref":"Gnat.Jetstream.PullConsumer.html#t:connection_option/0"},{"type":"type","title":"Gnat.Jetstream.PullConsumer.connection_options/0","doc":"Connection options used to connect the consumer to NATS server.","ref":"Gnat.Jetstream.PullConsumer.html#t:connection_options/0"},{"type":"type","title":"Gnat.Jetstream.PullConsumer.consumer/0","doc":"The pull consumer reference.","ref":"Gnat.Jetstream.PullConsumer.html#t:consumer/0"},{"type":"callback","title":"Gnat.Jetstream.PullConsumer.handle_message/2","doc":"Invoked to synchronously process a message pulled by the consumer.\nDepending on the value it returns, the acknowledgement is or is not sent.","ref":"Gnat.Jetstream.PullConsumer.html#c:handle_message/2"},{"type":"callback","title":"ACK actions - Gnat.Jetstream.PullConsumer.handle_message/2","doc":"Possible ACK actions values explained:\n\n* `:ack` - acknowledges the message was handled and requests delivery of the next message to\n  the reply subject.\n* `:nack` - signals that the message will not be processed now and processing can move onto\n  the next message, NAK'd message will be retried.\n* `:term` - instructs the server to stop redelivery of a message without acknowledging it as\n  successfully processed.\n* `:noreply` - nothing is sent. You may send later asynchronously an ACK or NACK message using\n  the `Jetstream.ack/1` or `Jetstream.nack/1` and similar functions from `Jetstream` module.","ref":"Gnat.Jetstream.PullConsumer.html#c:handle_message/2-ack-actions"},{"type":"callback","title":"Example - Gnat.Jetstream.PullConsumer.handle_message/2","doc":"def handle_message(message, state) do\n      IO.inspect(message)\n      {:ack, state}\n    end","ref":"Gnat.Jetstream.PullConsumer.html#c:handle_message/2-example"},{"type":"callback","title":"Gnat.Jetstream.PullConsumer.init/1","doc":"Invoked when the server is started. `start_link/3` or `start/3` will block until it returns.\n\n`init_arg` is the argument term (second argument) passed to `start_link/3`.\n\nSee `c:Connection.init/1` for more details.","ref":"Gnat.Jetstream.PullConsumer.html#c:init/1"},{"type":"function","title":"Gnat.Jetstream.PullConsumer.start/3","doc":"Starts a `Jetstream.PullConsumer` process without links (outside of a supervision tree).\n\nSee `start_link/3` for more information.","ref":"Gnat.Jetstream.PullConsumer.html#start/3"},{"type":"function","title":"Gnat.Jetstream.PullConsumer.start_link/3","doc":"Starts a pull consumer linked to the current process with the given function.\n\nThis is often used to start the pull consumer as part of a supervision tree.\n\nOnce the server is started, the `c:init/1` function of the given `module` is called with\n`init_arg` as its argument to initialize the server. To ensure a synchronized start-up procedure,\nthis function does not return until `c:init/1` has returned.\n\nSee `GenServer.start_link/3` for more details.","ref":"Gnat.Jetstream.PullConsumer.html#start_link/3"},{"type":"behaviour","title":"Gnat.Server","doc":"A behavior for acting as a server for nats messages.\n\nYou can use this behavior in your own module and then use the `Gnat.ConsumerSupervisor` to listen for and respond to nats messages.","ref":"Gnat.Server.html"},{"type":"behaviour","title":"Example - Gnat.Server","doc":"defmodule MyApp.RpcServer do\n      use Gnat.Server\n\n      def request(%{body: _body}) do\n        {:reply, \"hi\"}\n      end\n\n      # defining an error handler is optional, the default one will just call Logger.error for you\n      def error(%{gnat: gnat, reply_to: reply_to}, _error) do\n        Gnat.pub(gnat, reply_to, \"Something went wrong and I can't handle your request\")\n      end\n    end","ref":"Gnat.Server.html#module-example"},{"type":"callback","title":"Gnat.Server.error/2","doc":"Called when an error occured during the `request/1`\n\nIf your `request/1` function returned `{:error, term}`, then the `term` you returned will be passed as the second argument.\nIf an exception was raised during your `request/1` function, then the exception will be passed as the second argument.\nIf your `request/1` function returned something other than the supported return types, then its return value will be passed as the second argument.","ref":"Gnat.Server.html#c:error/2"},{"type":"callback","title":"Gnat.Server.request/1","doc":"Called when a message is received from the broker","ref":"Gnat.Server.html#c:request/1"},{"type":"behaviour","title":"Gnat.Services.Server","doc":"A behavior for acting as a NATS service\n\nCreating a service with this behavior works almost exactly the same as `Gnat.Server`,\nwith the bonus that this service keeps track of requests, errors, processing time, and\nparticipates in service discovery and monitoring as defined by\nthe [NATS service protocol](https://github.com/nats-io/nats-architecture-and-design/blob/main/adr/ADR-32.md).","ref":"Gnat.Services.Server.html"},{"type":"behaviour","title":"Example - Gnat.Services.Server","doc":"defmodule MyApp.Service do\n      use Gnat.Services.Server\n\n      # Classic subject matching\n      def request(%{body: _body, topic: \"myservice.req\"}, _, _) do\n        {:reply, \"handled request\"}\n      end\n\n      # Can also match on endpoint or group\n      def request(msg, \"add\", \"calculator\") do\n        {:reply, \"42\"}\n      end\n\n      # defining an error handler is optional, the default one will just call Logger.error for you\n      def error(%{gnat: gnat, reply_to: reply_to}, _error) do\n        Gnat.pub(gnat, reply_to, \"Something went wrong and I can't handle your request\")\n      end\n    end","ref":"Gnat.Services.Server.html#module-example"},{"type":"type","title":"Gnat.Services.Server.endpoint_configuration/0","doc":"Each service configuration must contain at least one endpoint. Endpoints can manually specify their\nsubscription subjects or they can be derived from the endpoint name.\n\n* `subject` - A specific subject for this endpoint to listen on. If this is not provided, then the endpoint name will be used.\n* `name` - The required name of the endpoint\n* `group_name` - An optional group to which this endpoint belongs\n* `queue_group` - A queue group for this endpoint's subscription. If not supplied, \"q\" will be used (indicated by protocol spec).\n* `metadata` - An optional string->string map containing metadata for this endpoint","ref":"Gnat.Services.Server.html#t:endpoint_configuration/0"},{"type":"callback","title":"Gnat.Services.Server.error/2","doc":"Called when an error occured during the `request/1`. Automatically increments the error count\nand processing time stats for this service.\n\nIf your `request/1` function returned `{:error, term}`, then the `term` you returned will be passed as the second argument.\nIf an exception was raised during your `request/1` function, then the exception will be passed as the second argument.\nIf your `request/1` function returned something other than the supported return types, then its return value will be passed as the second argument.","ref":"Gnat.Services.Server.html#c:error/2"},{"type":"callback","title":"Gnat.Services.Server.request/3","doc":"Called when a message is received from the broker. The endpoint on which the message arrived\nis always supplied. If the endpoint is a member of a group, the group name will also be\nprovided.\n\nAutomatically increments the request time and processing time stats for this service.","ref":"Gnat.Services.Server.html#c:request/3"},{"type":"type","title":"Gnat.Services.Server.service_configuration/0","doc":"Service configuration is provided as part of the consumer supervisor settings in the `service_definition` field.\nYou can specify _either_ the `subscription_topics` field for a regluar server or the `service_definition` field\nfor a new NATS service.\n\n* `name` - The name of the service. Needs to conform to the rules for NATS service names\n* `version` - A required version number (w/out \"v\" prefix) conforming to semver rules\n* `queue_group` - An optional queue group for service subscriptions. If left off, \"q\" will be used.\n* `description` - An optional description of the service\n* `metadata` - An optional string->string map of service metadata\n* `endpoints` - A required list of service endpoints. All services must have at least one endpoint","ref":"Gnat.Services.Server.html#t:service_configuration/0"},{"type":"extras","title":"README","doc":"[![hex.pm](https://img.shields.io/hexpm/v/gnat.svg)](https://hex.pm/packages/gnat)\n[![hex.pm](https://img.shields.io/hexpm/dt/gnat.svg)](https://hex.pm/packages/gnat)\n[![hex.pm](https://img.shields.io/hexpm/l/gnat.svg)](https://hex.pm/packages/gnat)\n[![github.com](https://img.shields.io/github/last-commit/nats-io/nats.ex.svg)](https://github.com/nats-io/nats.ex)\n\n![NATS](https://nats.io/img/logos/nats-horizontal-color.png)\n\n# Gnat\n\nA [nats.io](https://nats.io/) client for Elixir.\nThe goals of the project are resiliency, performance, and ease of use.\n\n> Hex documentation available here: https://hex.pm/packages/gnat","ref":"readme.html"},{"type":"extras","title":"Usage - README","doc":"``` elixir\n{:ok, gnat} = Gnat.start_link(%{host: \"127.0.0.1\", port: 4222})\n# Or if the server requires TLS you can start a connection with:\n# {:ok, gnat} = Gnat.start_link(%{host: \"127.0.0.1\", port: 4222, tls: true})\n\n{:ok, subscription} = Gnat.sub(gnat, self(), \"pawnee.*\")\n:ok = Gnat.pub(gnat, \"pawnee.news\", \"Leslie Knope recalled from city council (Jammed)\")\nreceive do\n  {:msg, %{body: body, topic: \"pawnee.news\", reply_to: nil}} ->\n    IO.puts(body)\nend\n```","ref":"readme.html#usage"},{"type":"extras","title":"Authentication - README","doc":"``` elixir\n# with user and password\n{:ok, gnat} = Gnat.start_link(%{host: \"127.0.0.1\", port: 4222, username: \"joe\", password: \"123\", auth_required: true})\n\n# with token\n{:ok, gnat} = Gnat.start_link(%{host: \"127.0.0.1\", port: 4222, token: \"secret\", auth_required: true})\n\n# with an nkey seed\n{:ok, gnat} = Gnat.start_link(%{host: \"127.0.0.1\", port: 4222, nkey_seed: \"SUAM...\", auth_required: true})\n\n# with decentralized user credentials (JWT)\n{:ok, gnat} = Gnat.start_link(%{host: \"127.0.0.1\", port: 4222, nkey_seed: \"SUAM...\", jwt: \"eyJ0eX...\", auth_required: true})\n\n# connect to NGS with JWT\n{:ok, gnat} = Gnat.start_link(%{host: \"connect.ngs.global\", tls: true, jwt: \"ey...\", nkey_seed: \"SUAM...\"})\n```","ref":"readme.html#authentication"},{"type":"extras","title":"TLS Connections - README","doc":"[NATS Server](https://github.com/nats-io/nats-server) is often configured to accept or require TLS connections.\nIn order to connect to these clusters you'll want to pass some extra TLS settings to your `Gnat` connection.\n\n``` elixir\n# using a basic TLS connection\n{:ok, gnat} = Gnat.start_link(%{host: \"127.0.0.1\", port: 4222, tls: true})\n\n# Passing a Client Certificate for verification\n{:ok, gnat} = Gnat.start_link(%{tls: true, ssl_opts: [certfile: \"client-cert.pem\", keyfile: \"client-key.pem\"]})\n```","ref":"readme.html#tls-connections"},{"type":"extras","title":"Resiliency - README","doc":"If you would like to stay connected to a cluster of nats servers, you should consider using `Gnat.ConnectionSupervisor` .\nThis can be added to your supervision tree in your project and will handle automatically re-connecting to the cluster.\n\nFor long-lived subscriptions consider using `Gnat.ConsumerSupervisor` .\nThis can also be added to your supervision tree and use a supervised connection to re-establish a subscription.\nIt also handles details like handling each message in a supervised process so you isolate failures and get OTP logs when an unexpected error occurs.","ref":"readme.html#resiliency"},{"type":"extras","title":"Services - README","doc":"If you supply a module that implements the `Gnat.Services.Server` behavior and the `service_definition` \nconfiguration field to a `Gnat.ConsumerSupervisor`, then this client will automatically take care\nof exposing the service to discovery, responding to pings, and maintaining and exposing statistics like request and error counts, and processing times.","ref":"readme.html#services"},{"type":"extras","title":"Instrumentation - README","doc":"Gnat uses [telemetry](https://hex.pm/packages/telemetry) to make instrumentation data available to clients.\nIf you want to record metrics around the number of messages or latency of message publishes, subscribes, requests, etc you can do the following in your project:\n\n``` elixir\niex(1)> metrics_function = fn(event_name, measurements, event_meta, config) ->\n  IO.inspect([event_name, measurements, event_meta, config])\n  :ok\nend\n#Function<4.128620087/4 in :erl_eval.expr/5>\niex(2)> names = [[:gnat, :pub], [:gnat, :sub], [:gnat, :message_received], [:gnat, :request], [:gnat, :unsub]]\n[\n  [:gnat, :pub],\n  [:gnat, :sub],\n  [:gnat, :message_received],\n  [:gnat, :request],\n  [:gnat, :unsub],\n  [:gnat, :service_request],\n  [:gnat, :service_error]\n]\niex(3)> :telemetry.attach_many(\"my listener\", names, metrics_function, %{my_config: true})\n:ok\niex(4)> {:ok, gnat} = Gnat.start_link()\n{:ok, #PID<0.203.0>}\niex(5)> Gnat.sub(gnat, self(), \"topic\")\n[[:gnat, :sub], %{latency: 128000}, %{topic: \"topic\"}, %{my_config: true}]\n{:ok, 1}\niex(6)> Gnat.pub(gnat, \"topic\", \"ohai\")\n[[:gnat, :pub], %{latency: 117000}, %{topic: \"topic\"}, %{my_config: true}]\n[[:gnat, :message_received], %{count: 1}, %{topic: \"topic\"}, %{my_config: true}]\n:ok\n```\n\nThe `pub` , `sub` , `request` , and `unsub` events all report the latency of those respective calls.\nThe `message_received` event reports a number of messages like `%{count: 1}` because there isn't a good latency metric to report. Any microservices managed by a consumer supervisor will also report `service_request` and `service_error`. In addition to the `:topic` metadata, microservices will also include `:endpoint` and `:group` (which can be `nil`) in their telemetry reports.\n\nAll of the events (except `unsub` ) include metadata with a `:topic` key so you can split your metrics by topic.","ref":"readme.html#instrumentation"},{"type":"extras","title":"Benchmarks - README","doc":"Part of the motivation for building this library is to get better performance.\nTo this end, there is a `bench` branch on this project which includes a `server.exs` and `client.exs` that can be used for benchmarking various scenarios.\n\nAs of this commit, the [latest benchmark on a 16-core server](https://gist.github.com/mmmries/08fe44fdd47a6f8838936f41170f270a) shows that you can make 170k+ req/sec or up to 192MB/sec.\n\nThe `bench/*.exs` files also contain some straight-line single-CPU performance tests.\nAs of this commit my 2018 MacBook pro shows.\n\n|               | ips      | average   | deviation | median |\n| ------------- | -------- | --------- | --------- | ------ |\n| parse-128     | 487.67 K | 2.19 μs   | ±1701.54% | 2 μs   |\n| pub - 128     | 96.37 K  | 10.38 μs  | ±102.94%  | 10 μs  |\n| req-reply-128 | 8.32 K   | 120.16 μs | ±23.68%   | 114 μs |","ref":"readme.html#benchmarks"},{"type":"extras","title":"Development - README","doc":"Before running the tests make sure you have a locally running copy of `nats-server` ([installation instructions](https://docs.nats.io/running-a-nats-service/introduction/installation)). By default, tests are run with no authentication. Make sure your NATS configuration contains no users, or has an account with [no_auth_user](https://docs.nats.io/running-a-nats-service/configuration/securing_nats/accounts#no-auth-user) explicitly enabled.\n\nWe currently use version `2.10.24` in CI, but anything higher than `2.2.0` should be fine.\nVersions from `0.9.6` up to `2.2.0` should work fine for everything except header support.\nMake sure to enable jetstream with the `nats-server -js` argument and you might also want to enable debug and verbose logging if you're trying to understand the messages being sent to/from nats (ie `nats-server -js -D -V`).\nThe typical `mix test` will run all the basic unit tests.\n\nYou can also run the `multi_server` set of tests that test connectivity to different\n`nats-server` configurations. You can run these with `mix test --only multi_server` .\nThe tests will tell you how to start the different configurations.\n\nThere are also some property-based tests that generate a lot of test cases.\nYou can tune how many test cases by setting the environment variable `N=200 mix test --only property` (default it 100).\n\nFor more details you can look at how Github runs these things in the CI flow.","ref":"readme.html#development"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"1.10 - Changelog","doc":"* Clarify authentication setup during test by @davydog187 in #187\n* Test on Elixir 1.18 and NATS 2.10.24 by @davydog187 in #188\n* Gnat.Jetstream.API.KV.info/3 by @davydog187 in #189\n* Remove function_exported? check for Keyword.validate!/2 by @davydog187 in #186\n* Tiny optimization to KV.list_buckets/1 by @davydog187 in #185\n* make KV-watcher emit :key_added events when the message has a header by @rixmann in #191\n* add :compression to stream attributes by @rixmann in #192\n* fix: unknown field domain in Stream.create (#194) by @c0deaddict\n* feat: add jetstream message metadata helper (#197) by @c0deaddict\n* fix: deliver policy (#196) by @c0deaddict","ref":"changelog.html#1-10"},{"type":"extras","title":"1.9 - Changelog","doc":"* Housecleaning by @mmmries in #176\n  * switch to charlist sigils\n  * update to newest nkeys\n  * require elixir 1.14 and erlang 25+\n* Fix incorrect useage of charlist by @davydog187 in #179\n* Soft deprecate is_kv_bucket_stream?/1 in favor of kv_bucket_stream?/1 by @davydog187 in #183\n* Clean up examples in KV by @davydog187 in #181\n* Document options for Gnat.Jetstream.API.KV by @davydog187 in #180","ref":"changelog.html#1-9"},{"type":"extras","title":"1.8 - Changelog","doc":"* Integrated the jetstream functionality into this client directly https://github.com/nats-io/nats.ex/pull/146\n* Add ability to list KV buckets https://github.com/nats-io/nats.ex/pull/152\n* Improve CI Reliability https://github.com/nats-io/nats.ex/pull/154\n* Bugfix to treat no streams as an empty list rather than a null https://github.com/nats-io/nats.ex/pull/155\n* Added supported for `allow_direct` and `mirror_direct` attributes of streams https://github.com/nats-io/nats.ex/pull/161\n* Added support for `discard_new_per_subject` attribute of streams https://github.com/nats-io/nats.ex/pull/163\n* Added support for `Object.list_buckets` https://github.com/nats-io/nats.ex/pull/169","ref":"changelog.html#1-8"},{"type":"extras","title":"1.7 - Changelog","doc":"* Added support for the NATS [services API](https://github.com/nats-io/nats.go/blob/main/micro/README.md), letting developers participate in service discovery and stats https://github.com/nats-io/nats.ex/pull/141\n * A bugfix to remove the queue_group from a service config and some optimization for the services API https://github.com/nats-io/nats.ex/pull/145","ref":"changelog.html#1-7"},{"type":"extras","title":"1.6 - Changelog","doc":"* added the `no_responders` behavior https://github.com/nats-io/nats.ex/pull/137","ref":"changelog.html#1-6"},{"type":"extras","title":"1.5 - Changelog","doc":"* add the `inbox_prefix` option https://github.com/nats-io/nats.ex/pull/121\n* add the `Gnat.server_info/1` function https://github.com/nats-io/nats.ex/pull/124\n* fix header parsing issue https://github.com/nats-io/nats.ex/pull/125","ref":"changelog.html#1-5"},{"type":"extras","title":"1.4 - Changelog","doc":"* add the `Gnat.request_multi/4` function https://github.com/nats-io/nats.ex/pull/120\n* add elixir 1.13 to the test matrix","ref":"changelog.html#1-4"},{"type":"extras","title":"1.3 - Changelog","doc":"* adding support for sending and receiving headers https://github.com/nats-io/nats.ex/pull/116","ref":"changelog.html#1-3"},{"type":"extras","title":"1.2 - Changelog","doc":"* `Gnat.Server` behaviour with support in the `ConsumerSupervisor` https://github.com/nats-io/nats.ex/compare/1b1adc85e4b28231218ef87c7fc3445fce854377...b24a7e14325b51fbb93fde7e3d891d18b4fa8afb\n* avoid logging sensitive credentials https://github.com/nats-io/nats.ex/pull/105\n* deprecate Gnat.ping, improved typespecs https://github.com/nats-io/nats.ex/pull/103 \n* relax the version constraint on nimble_parsec https://github.com/nats-io/nats.ex/issues/112","ref":"changelog.html#1-2"},{"type":"extras","title":"1.1 - Changelog","doc":"* add support for nkeys and NGS https://github.com/nats-io/nats.ex/pull/101\n* Fix supervisor ConsumerSuperivsor crash https://github.com/nats-io/nats.ex/pull/96","ref":"changelog.html#1-1"},{"type":"extras","title":"1.0 - Changelog","doc":"* Make supervisors officially supported https://github.com/nats-io/nats.ex/pull/96","ref":"changelog.html#1-0"},{"type":"extras","title":"0.7.0 - Changelog","doc":"* update to telemetry 0.4 https://github.com/nats-io/nats.ex/pull/86 and https://github.com/nats-io/nats.ex/pull/87\n* support for token authentication https://github.com/nats-io/nats.ex/pull/92\n* support elixir 1.9 https://github.com/nats-io/nats.ex/pull/93","ref":"changelog.html#0-7-0"},{"type":"extras","title":"0.6.0 - Changelog","doc":"* Dropped support for Erlang < 19 and Elixir <= 1.5\n* Added Telemetry to the project (thanks @rubysolo)\n* Switched to nimble_parsec for parsing\n  * Updated benchmarking/performance information. We can now do 170k requests per second on a 16-core server.\n* Fixed a bug around re-subscribing for the `ConsumerSupervisor`\n* Pass `sid` when delivering message (thanks @entone)\n* Documentation fixes from @deini and @johannestroeger","ref":"changelog.html#0-6-0"},{"type":"extras","title":"0.5.0 - Changelog","doc":"* Dropped support for Elixir 1.4 and OTP 18 releases. You will need to use Elixir 1.5+ and OTP 19+.\n* Switched to running our tests against gnatsd `1.3.0`","ref":"changelog.html#0-5-0"},{"type":"extras","title":"Overview","doc":"# Overview\n\n[Jetstream](https://docs.nats.io/nats-concepts/jetstream) is a distributed persistence system\nbuilt-in to [NATS](https://nats.io/). It provides a streaming system that lets you capture streams\nof events from various sources and persist these into persistent stores, which you can immediately\nor later replay for processing.\n\nThis library exposes interfaces for publishing, consuming and managing Jetstream services. It builds\non top of [Gnat](https://hex.pm/packages/gnat), the officially supported Elixir client for NATS.\n\n* [Let's get Jetstream up and running](./getting_started.md)\n* [Using Broadway with Jetstream](../guides/broadway.md)\n* [Pull Consumer API](`Gnat.Jetstream.PullConsumer`)\n* [Create, update and delete Jetstream streams and consumers via Elixir](../guides/managing.md)","ref":"overview.html"},{"type":"extras","title":"Getting Started","doc":"# Getting Started\n\nIn this guide, we're going to learn how to install Jetstream in your project and start consuming\nmessages from your streams.","ref":"getting_started.html"},{"type":"extras","title":"Starting Jetstream - Getting Started","doc":"The following Docker Compose file will do the job:\n\n```yaml\nversion: \"3\"\nservices:\n  nats:\n    image: nats:latest\n    command:\n      - -js\n    ports:\n      - 4222:4222\n```\n\nSave this snippet as `docker-compose.yml` and run the following command:\n\n```shell\ndocker compose up -d\n```\n\nLet's also create Jetstream stream where we will publish our hello world messages:\n\n```shell\nnats stream add HELLO --subjects=\"greetings\"\n```\n\n> #### Tip {: .tip}\n>\n> You can also manage Jetstream streams and consumers via Elixir. You can see more details in\n> [this guide](../guides/managing.md).","ref":"getting_started.html#starting-jetstream"},{"type":"extras","title":"Adding Jetstream and Gnat to an application - Getting Started","doc":"To start off with, we'll generate a new Elixir application by running this command:\n\n```\nmix new hello_jetstream --sup\n```\n\nWe need to have [a supervision tree](http://elixir-lang.org/getting-started/mix-otp/supervisor-and-application.html)\nup and running in your app, and the `--sup` option ensures that.\n\nTo add Jetstream to this application, you need to add [Jetstream](https://hex.pm/packages/jetstream)\nand [Gnat](https://hex.pm/packages/gnat) libraries to your `deps` definition in our `mix.exs` file.\n**Fill exact version requirements from each package Hex.pm pages.**\n\n```elixir\ndefp deps do\n  [\n    {:gnat, ...},\n    {:jetstream, ...}\n  ]\nend\n```\n\nTo install these dependencies, we will run this command:\n\n```shell\nmix deps.get\n```\n\nNow let's connect to our NATS server. To do this, you need to start `Gnat.ConnectionSupervisor`\nunder our application's supervision tree. Add following to `lib/hello_jetstream/application.ex`:\n\n```elixir\ndef start(_type, _args) do\n  children = [\n    ...\n\n    # Create NATS connection\n    {Gnat.ConnectionSupervisor,\n      %{\n        name: :gnat,\n        connection_settings: [\n          %{host: \"localhost\", port: 4222}\n        ]\n      }},\n  ]\n\n  ...\n```\n\nThis piece of configuration will start Gnat processes that connect to the NATS server and allow\npublishing and subscribing to any subjects. Jetstream operates using plain NATS subjects which\nfollow specific naming and message format conventions.\n\nLet's now create a _pull consumer_ which will subscribe a specific Jetstream stream and print\nincoming messages to standard output.","ref":"getting_started.html#adding-jetstream-and-gnat-to-an-application"},{"type":"extras","title":"Creating a pull consumer - Getting Started","doc":"Jetstream requires us to allocate a view/cursor of the stream that our consumer will operate on.\nIn Jetstream terminology, this view is called a _consumer_ (Funnily enough we've just implemented\na consumer in our code, coincidence?). [Jetstream](https://docs.nats.io/nats-concepts/jetstream/consumers)\n[documentation](https://docs.nats.io/nats-concepts/jetstream/consumers/example_configuration)\noffers great insights on benefits of having this separate concept so we won't duplicate work here.\n\nJetstream offers two stream consuming modes: _push_ and _pull_.\n\nIn _push_ mode, Jetstream will simply send messages to selected consumers immediately when they are\nreceived. This approach does offer congestion control, so it is not recommended for high-volume\nand/or reliability sensitive streams. You do not really need this library to implement push\nconsumer because all building blocks are in `Gnat` library. You can read more about push consumers\nin [this guide](../guides/push_based_consumer.md).\n\nOn the other hand, in _pull_ mode consumers ask Jetstream for more messages when they are ready\nto process them. This is the recommended approach for most use cases and we will proceed with it\nin this guide.\n\n> #### This is just a brief outline {: .tip}\n>\n> For more details about differences between consumer modes, consult\n> [Jetstream documentation](https://docs.nats.io/nats-concepts/jetstream/consumers).\n\nLet's create a pull consumer module within our application at\n`lib/hello_jetstream/logger_pull_consumer.ex`:\n\n```elixir\ndefmodule HelloJetstream.LoggerPullConsumer do\n  use Jetstream.PullConsumer\n\n  def start_link([]) do\n    Jetstream.PullConsumer.start_link(__MODULE__, [])\n  end\n\n  @impl true\n  def init([]) do\n    {:ok, nil, connection_name: :gnat, stream_name: \"HELLO\", consumer_name: \"LOGGER\"}\n  end\n\n  @impl true\n  def handle_message(message, state) do\n    IO.inspect(message)\n    {:ack, state}\n  end\nend\n```\n\nPull Consumer is a regular `GenServer` and it takes a reference to `Gnat.ConnectionSupervisor`\nalong with names of Jetstream stream and consumer as options passed to\n`Jetstream.PullConsumer.start*` functions. These options are passed as keyword list in third element\nof tuple returned from the `c:Jetstream.PullConsumer.init/1` callback.\n\nThe only required callbacks are well known gen server's `c:Jetstream.PullConsumer.init/1` and\n`c:Jetstream.PullConsumer.handle_message/2`, which takes new message as its first argument and\nis expected to return an _ACK action_ instructing underlying process loop what to do with this\nmessage. Here we are asking it to automatically send for us an ACK message back to Jetstream.\n\nLet's now create a consumer in our NATS server. We will call it `LOGGER` as we plan to let it simply\nlog everything published to the stream.\n\n```shell\nnats consumer add --pull --deliver=all HELLO LOGGER\n```\n\nNow, let's start our pull consumer under application's supervision tree.\n\n```elixir\ndef start(_type, _args) do\n  children = [\n    ...\n\n    # Jetstream Pull Consumer\n    HelloJetstream.LoggerPullConsumer,\n  ]\n\n  ...\n```\n\nLet's now publish some messages to our `HELLO` stream, so something will be waiting for our\napplication to be read when it starts.","ref":"getting_started.html#creating-a-pull-consumer"},{"type":"extras","title":"Publishing messages to streams - Getting Started","doc":"Jetstream listens on regular NATS subjects, so publishing messages is dead simple with `Gnat.pub/3`:\n\n```elixir\nGnat.pub(:gnat, \"greetings\", \"Hello World\")\n```\n\nOr via NATS CLI:\n\n```shell\nnats pub greetings \"Hello World\"\n```\n\nThat's it! When you run your app, you should see your messages being read by your application.","ref":"getting_started.html#publishing-messages-to-streams"},{"type":"extras","title":"Managing Streams and Consumers","doc":"# Managing Streams and Consumers\n\nJetstream provides a JSON API for managing streams and consumers.\nThis library exposes this API via interactions with the `Jetstream.Api.Stream` and `Jetstream.Api.Consumer` modules.\n\nThese modules act as native wrappers for the API and do not attempt to simplify any of the common use-cases.\nAs this library matures we may introduce a separate layer of functions to handle these scenarios, but for now our aim is to provide full access to the Jetstream API.","ref":"managing.html"},{"type":"extras","title":"Push based consumer","doc":"# Push based consumer\n\n```elixir\n# Start a nats server with jetstream enabled and default configs\n# Now run the following snippets in an IEx terminal\nalias Jetstream.API.{Consumer,Stream}\n\n# Setup a connection to the nats server and create the stream/consumer\n# This is the equivalent of these two nats cli commands\n#   nats stream add TEST --subjects=\"greetings\" --max-msgs=-1 --max-msg-size=-1 --max-bytes=-1 --max-age=-1 --storage=file --retention=limits --discard=old\n#   nats consumer add TEST TEST --target consumer.greetings --replay instant --deliver=all --ack all --wait=5s --filter=\"\" --max-deliver=10\n{:ok, connection} = Gnat.start_link()\nstream = %Stream{name: \"TEST\", subjects: [\"greetings\"]}\n{:ok, _response} = Stream.create(connection, stream)\nconsumer = %Consumer{stream_name: \"TEST\", name: \"TEST\", deliver_subject: \"consumer.greetings\", ack_wait: 5_000_000_000, max_deliver: 10}\n{:ok, _response} = Consumer.create(connection, consumer)\n\n# Setup Consuming Function\ndefmodule Subscriber do\n  def handle(msg) do\n    IO.inspect(msg)\n    case msg.body do\n      \"hola\" -> Jetstream.ack(msg)\n      \"bom dia\" -> Jetstream.nack(msg)\n      _ -> nil\n    end\n  end\nend\n\n# normally you would add the `ConnectionSupervisor` and `ConsumerSupervisor` to your supervisrion tree\n# here we start them up manually in an IEx session\n{:ok, _pid} = Gnat.ConnectionSupervisor.start_link(%{\n  name: :gnat,\n  backoff_period: 4_000,\n  connection_settings: [\n    %{}\n  ]\n})\n{:ok, _pid} = Gnat.ConsumerSupervisor.start_link(%{\n  connection_name: :gnat,\n  consuming_function: {Subscriber, :handle},\n  subscription_topics: [\n    %{topic: \"consumer.greetings\"}\n  ]\n})\n\n# now publish some messages into the stream\nGnat.pub(:gnat, \"greetings\", \"hello\") # no ack will be sent back, so you'll see this message received 10 times with a 5sec pause between each one\nGnat.pub(:gnat, \"greetings\", \"hola\") # an ack is sent back so this will only be received once\nGnat.pub(:gnat, \"greetings\", \"bom dia\") # a -NAK is sent back so you'll see this received 10 times very quickly\n```","ref":"push_based_consumer.html"},{"type":"extras","title":"Using Broadway with Jetstream","doc":"# Using Broadway with Jetstream\n\nBroadway is a library which allows building concurrent and multi-stage data ingestion and data\nprocessing pipelines with Elixir easily. You can learn about it more in\n[Broadway documentation](https://hexdocs.pm/broadway/introduction.html).\n\nJetstream library comes with tools necessary to use NATS Jetstream with Broadway.","ref":"broadway.html"},{"type":"extras","title":"Getting started - Using Broadway with Jetstream","doc":"In order to use Broadway with NATS Jetstream you need to:\n\n1. Setup a NATS Server with JetStream turned on\n2. Create stream and consumer on NATS server\n3. Configure Gnat connection in your Elixir project\n4. Configure your project to use Broadway\n\nIn this guide, we are going to focus on the fourth point. To learn how to start Jetstream locally\nwith Docker Compose and then add Gnat and Jetstream to your application, see the Starting Jetstream\nsection in [Getting Started guide](../introduction/getting_started.md).","ref":"broadway.html#getting-started"},{"type":"extras","title":"Adding Broadway to your application - Using Broadway with Jetstream","doc":"Once we have NATS with JetStream running and the stream and consumer we are going to use are\ncreated, we can proceed to adding Broadway to our project. First, put `:broadway` to the list of\ndependencies in `mix.exs`.\n\n```elixir\ndefp deps do\n  [\n    ...\n    {:broadway, ...version...},\n    ...\n  ]\nend\n```\n\nVisit [Broadway page on Hex.pm](https://hex.pm/packages/broadway) to check for current version\nto put in `deps`.\n\nTo install the dependencies, run:\n\n```shell\nmix deps.get\n```","ref":"broadway.html#adding-broadway-to-your-application"},{"type":"extras","title":"Defining the pipeline configuration - Using Broadway with Jetstream","doc":"The next step is to define your Broadway module. We need to implement three functions in order\nto define a Broadway pipeline: `start_link/1`, `handle_message/3` and `handle_batch/4`.\nLet's create `start_link/1` first:\n\n```elixir\ndefmodule MyBroadway do\n  use Broadway\n\n  alias Broadway.Message\n\n  def start_link(_opts) do\n    Broadway.start_link(\n      __MODULE__,\n      name: MyBroadwayExample,\n      producer: [\n        module: {\n          OffBroadway.Jetstream.Producer,\n          connection_name: :gnat,\n          stream_name: \"TEST_STREAM\",\n          consumer_name: \"TEST_CONSUMER\"\n        },\n        concurrency: 10\n      ],\n      processors: [\n        default: [concurrency: 10]\n      ],\n      batchers: [\n        default: [\n          concurrency: 5,\n          batch_size: 10,\n          batch_timeout: 2_000\n        ]\n      ]\n      ...\n    )\n  end\n\n  ...callbacks..\nend\n```\n\nAll `start_link/1` does is just delegating to `Broadway.start_link/2`.\n\nTo understand what all these options mean and to learn about other possible settings, visit\n[Broadway documentation](https://hexdocs.pm/broadway/Broadway.html).\n\nThe part that interests us the most in this guide is the `producer.module`. Here we're choosing\n`OffBroadway.Jetstream.Producer` as the producer module and passing the connection options,\nsuch as Gnat process name and stream name. For full list of available options, visit\n[Producer](`OffBroadway.Jetstream.Producer`) documentation.","ref":"broadway.html#defining-the-pipeline-configuration"},{"type":"extras","title":"Implementing Broadway callbacks - Using Broadway with Jetstream","doc":"Broadway requires some callbacks to be implemented in order to process messages. For full list\nof available callbacks visit\n[Broadway documentation](https://hexdocs.pm/broadway/Broadway.html#callbacks).\n\nA simple example:\n\n```elixir\ndefmodule MyBroadway do\n  use Broadway\n\n  alias Broadway.Message\n\n  ...start_link...\n\n  def handle_message(_processor_name, message, _context) do\n    message\n    |> Message.update_data(&process_data/1)\n    |> case do\n      \"FOO\" -> Message.configure_ack(on_success: :term)\n      \"BAR\" -> Message.configure_ack(on_success: :nack)\n      message -> message\n    end\n  end\n\n  defp process_data(data) do\n    String.upcase(data)\n  end\n\n  def handle_batch(_, messages, _, _) do\n    list = messages |> Enum.map(fn e -> e.data end)\n    IO.puts(\"Got a batch: #{inspect(list)}. Sending acknowledgements...\")\n    messages\n  end\n```\n\nFirst, in `handle_message/3` we update our messages' data individually by converting them to\nuppercase. Then, in the same callback, we're changing the success ack option of the message\nto `:term` if its content is `\"FOO\"` or to `:nack` if the message is `\"BAR\"`. In the end we\nprint each batch in `handle_batch/4`. It's not quite useful but should be enough for this\nguide.","ref":"broadway.html#implementing-broadway-callbacks"},{"type":"extras","title":"Running the Broadway pipeline - Using Broadway with Jetstream","doc":"Once we have our pipeline fully defined, we need to add it as a child in the supervision tree.\nMost applications have a supervision tree defined at `lib/my_app/application.ex`.\n\n```elixir\nchildren = [\n  {MyBroadway, []}\n]\n\nSupervisor.start_link(children, strategy: :one_for_one)\n```\n\nYou can now test the pipeline. Let's start the application:\n\n```shell\niex -S mix\n```\n\nUse Gnat API to send messages to your stream:\n\n```elixir\nGnat.pub(:gnat, \"test_subject\", \"foo\")\nGnat.pub(:gnat, \"test_subject\", \"bar\")\nGnat.pub(:gnat, \"test_subject\", \"baz\")\n```\n\nBatcher should then print:\n\n```\nGot a batch: [\"FOO\", \"BAR\", \"BAZ\"]. Sending acknowledgements...\n```","ref":"broadway.html#running-the-broadway-pipeline"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}